const DEFAULT_LISTENER_FROM_PARTNER_PORT = 25614;

const express = require('express');

const config = require('komodo-sdk/config');
const logger = require('komodo-sdk/logger');

const middlewareCommon = require('../middlewares/common');

const routerInquiry = require('./routers/inquiry');
const routerPay = require('./routers/pay');
const routerTopup = require('./routers/topup');
const routerTrxStatus = require('./routers/trx-status');

const app = express();

app.use(express.json({ extended: true }));
app.use(express.urlencoded({ extended: true }));
app.use(middlewareCommon);

app.use('/inquiry', routerInquiry);
app.use('/pay', routerPay);
app.use('/topup', routerTopup);
app.use('/trx-status', routerTrxStatus);

app.use((req, res) => {
    const { xid } = res.locals;
    res.status(404).end(`KOMODO-HTTP-GET-X CENTER.\n404: Method not found.\n\nXID: ${xid}.\n`);
});

app.listen(config.listener.partner.port || DEFAULT_LISTENER_FROM_PARTNER_PORT, () => {
    logger.info(`Listen from partner request on port ${config.listener.partner.port}`);
}).on('error', (e) => {
    logger.error(`Can not listen request from partner on port ${config.listener.partner.port || DEFAULT_LISTENER_FROM_PARTNER_PORT}. ${e.toString()}`);
    process.exit(1);
});
