const express = require('express');
const moment = require('moment');

const logger = require('komodo-sdk/logger');

const coreapi = require('../../coreapi');

const router = express.Router();
module.exports = router;

async function pageIndex(req, res) {
    if (!req.body) req.body = {};

    if (!req.body.terminal_name && !req.query.terminal_name) {
        res.json({
            error: 'Parameter terminal_name tidak terdefinisi',
        });
        return;
    }

    if (!req.body.password && !req.query.password) {
        res.json({
            error: true,
            message: 'Parameter password tidak terdefinisi',
        });
        return;
    }

    if (!req.body.request_id && !req.query.request_id) {
        res.json({
            error: true,
            message: 'Parameter request_id tidak terdefinisi',
        });
        return;
    }

    const remoteIp = req.ip.replace(/^::ffff:/, '');
    const askerTerminalName = `${req.body.terminal_name || req.query.terminal_name}@${remoteIp}`;

    const coreResponse = await coreapi({
        xid: res.locals.xid,
        path: '/trx-status/view',
        qs: {
            asker_terminal_name: askerTerminalName,
            asker_terminal_password: req.body.password || req.query.password,
            request_id: req.body.request_id || req.query.request_id,
        },
    });

    if (!coreResponse || !coreResponse.status) {
        const msg = 'Status transaksi tidak dapat diketahui karena suatu kesalahan pada sistem';

        logger.warn(msg, {
            xid: res.locals.xid,
            ip: req.ip,
            terminal_name: req.body.terminal_name || req.query.terminal_name,
            request_id: req.body.request_id || req.query.request_id,
        });

        res.json({
            error: true,
            from_ip: remoteIp,
            terminal_name: req.body.terminal_name || req.query.terminal_name,
            full_terminal_name: askerTerminalName,
            password: req.body.password || req.query.password,
            message: msg,
        });
        return;
    }

    const trx = coreResponse && coreResponse.result ? {
        id: coreResponse.result.id,
        request_id: coreResponse.result.request_id,
        store_id: coreResponse.result.store_id,
        store_name: coreResponse.result.store_name,
        terminal_id: coreResponse.result.terminal_id,
        terminal_name: coreResponse.result.terminal_name,

        created: moment(coreResponse.result.created).format('YYYY-MM-DD HH:mm:ss'),
        modified: moment(coreResponse.result.modified).format('YYYY-MM-DD HH:mm:ss'),

        product_name: coreResponse.result.product_name,
        destination: coreResponse.result.destination,
        amount: coreResponse.result.amount,
        ending_balance: coreResponse.result.ending_balance,
        rc: coreResponse.result.rc,
        message: coreResponse.result.reply_message,
        sn: coreResponse.result.sn,
    }
        : null;

    const result = {
        error: coreResponse.status !== 'OK',
        from_ip: remoteIp,
        terminal_name: req.body.terminal_name || req.query.terminal_name,
        full_terminal_name: askerTerminalName,
        password: req.body.password || req.query.password,
        message: coreResponse.message,
        trx_found: Boolean(trx),
        trx,
        // original_trx: coreResponse.result,
    };

    res.json(result);

    logger.info('Partner request responded', { xid: res.locals.xid, processing_time_in_ms: new Date() - res.locals.x_http_request_ts, result });
}

router.get('/', pageIndex);
